#ifndef Minla_hh
#define Minla_hh


#include "Graph.hh"
#include "Layout.hh"




/**
	@short Static class that offers different methods to approximate the MINLA problem.
	@author Jordi Petit i Silvestre <jpetit@lsi.upc.es>
*/


class Minla
{
		
	public:

		/** Heuristic to solve Minla using the canonical layout. 
			Modifies the layout L and returns its LA cost. */
		static int canonical (const Graph& G, Layout& L);

		/** Heuristic to solve Minla using the spectral layout. 
			Modifies the layout L and returns its LA cost. */
		static int spectral (const Graph& G, Layout& L);

		/** Heuristic to solve Minla using a random layout. 
			Modifies the layout L and returns its LA cost. */
		static int random (const Graph& G, Layout& L);

		/** Solves Minla using the optimal layout. 
			Modifies the layout L and returns its LA cost.
			Uses backtracking and is thus very slow! */
		static int optimal (const Graph& G, Layout& L);
		
		/** Solves Minla using a greedy algorithm. 
			The initial ordering of the vertices is normal.
			Modifies the layout L and returns its LA cost.*/
		static int greedy_nor (const Graph& G, Layout& L);
		
		/** Solves Minla using a greedy algorithm. 
			The initial ordering of the vertices is random.
			Modifies the layout L and returns its LA cost. */
		static int greedy_rnd (const Graph& G, Layout& L);

		/** Solves Minla using a greedy algorithm. 
			The initial ordering of the vertices is giveb
			ny a random breadth search.
			Modifies the layout L and returns its LA cost. */
		static int greedy_rbs (const Graph& G, Layout& L);

		/** Solves Minla using a greedy algorithm. 
			The initial ordering of the vertices is giveb
			ny a breadth first search starting at a random node.
			Modifies the layout L and returns its LA cost. */
		static int greedy_bfs (const Graph& G, Layout& L);

		/** Solves Minla using a greedy algorithm. 
			The initial ordering of the vertices is giveb
			ny a depth first search starting at a random node.
			Modifies the layout L and returns its LA cost. */
		static int greedy_dfs (const Graph& G, Layout& L);

		static int metropolis_E (const Graph& G, Layout& L, double temp=-1);
		static int metropolis_2 (const Graph& G, Layout& L, double temp=-1);
		static int metropolis_3 (const Graph& G, Layout& L, double temp=-1);

		static int sssa (const Graph& G, Layout& L, 
			const double alpha=0.95,
			const int beta=12,
			const double t0=10,
			const double rho=0.9999);
		
};


#endif

